"""Celery task to bake charts to aws."""
from __future__ import absolute_import

import logging
import os

from boto3.session import Session
from celery import shared_task
from chartwerk.conf import settings as app_settings
from chartwerk.models import Chart
from django.contrib.staticfiles.templatetags.staticfiles import static
from django.template.loader import render_to_string
from django.utils.six.moves.urllib.request import urlopen

logger = logging.getLogger(__name__)


def get_chartwerk_bucket():
    session = Session(
        region_name='us-east-1',
        aws_access_key_id=app_settings.AWS_ACCESS_KEY_ID,
        aws_secret_access_key=app_settings.AWS_SECRET_ACCESS_KEY
    )
    s3 = session.resource('s3')
    return s3.Bucket(app_settings.AWS_BUCKET)


def render_local_static(static_file):
    """Render local static file to string to inject into template."""
    return urlopen(
        os.path.join(
            app_settings.DOMAIN,
            static(static_file)[1:]
        )
    ).read().decode('UTF-8')


def render_static_string(url):
    """Render static strings.

    Opens dependency files and renders them into a string for injection
    or returns the url string.
    """
    try:
        return {
            'script': urlopen(url).read().decode('UTF-8'),
            'inject': True
        }
    except:
        return {
            'url': url,
            'inject': False
        }


def build_dependencies(scripts, styles):
    """Build dependencies.

    Builds a dependencies dict that will prefer injecting CSS dependencies
    but can fall back to style tags.
    """
    dependencies = {
        'scripts': [
            {'url': script} for script in scripts
        ],
        'styles': [
            render_static_string(style) for style in styles
        ],
    }
    return dependencies


def cleaner(werk):
    """Clean chart data.

    Cleans off some unnecessary data from the payload, needed only when editing
    a chart.
    """
    werk.data['ui']['rawData'] = None
    werk.data['template']['description'] = None
    werk.data['scripts'] = None
    return werk


@shared_task
def write_to_aws(pk):
    """Write to AWS S3 bucket.

    Creates dependency file context for injection, renders template as string
    and ships baked charts to AWS.
    """
    bucket = get_chartwerk_bucket()

    if not bucket:
        return

    werk = Chart.objects.get(pk=pk)

    try:
        werk.client = {
            'scripts': render_local_static('chartwerk/js/client.bundle.js'),
            'styles': render_local_static('chartwerk/css/client.css'),
        }
        werk.dependencies = build_dependencies(
            scripts=werk.data['scripts']['dependencies']['scripts'],
            styles=werk.data['scripts']['dependencies']['styles']
        )
        werk.scripts = werk.data['scripts']
        werk = cleaner(werk)
        # DOUBLE-WIDE
        werk.data['ui']['size'] = 'double'
        key = os.path.join(
            app_settings.AWS_PATH,
            "{}.html".format(werk.slug)
        )
        bucket.Object(key).put(
            Body=render_to_string(
                'chartwerk/bake.html', {'werk': werk}),
            ContentType='text/html',
            CacheControl=app_settings.CACHE_HEADER,
            ACL='public-read',
        )
        # SINGLE-WIDE
        werk.data['ui']['size'] = 'single'
        key = os.path.join(
            app_settings.AWS_PATH,
            "{}_single.html".format(werk.slug)
        )
        bucket.Object(key).put(
            Body=render_to_string(
                'chartwerk/bake.html', {'werk': werk}),
            ContentType='text/html',
            CacheControl=app_settings.CACHE_HEADER,
            ACL='public-read',
        )
    except Exception:
        logging.exception("AWS write error")



import json
import os
import re
from importlib import import_module

from chartwerk.conf import settings as app_settings
from chartwerk.models import Chart, FinderQuestion, Template, TemplateProperty
from chartwerk.serializers import (ChartEmbedSerializer, ChartSerializer,
                                   FinderQuestionSerializer,
                                   TemplatePropertySerializer,
                                   TemplateSerializer)
from django.conf import settings
from django.core.urlresolvers import NoReverseMatch, reverse
from django.http import (HttpResponseBadRequest, HttpResponseNotFound,
                         JsonResponse)
from django.urls import resolve
from django.utils.decorators import method_decorator
from django.utils.six.moves.urllib.parse import urlparse
from django.views.generic import DetailView, ListView, TemplateView
from rest_framework import viewsets


def import_class(val):
    """Attempt to import a class from a string representation.

    Pattern borrowed from Django REST Framework.
    See rest_framework/settings.py#L170-L182
    """
    try:
        parts = val.split('.')
        module_path, class_name = '.'.join(parts[:-1]), parts[-1]
        module = import_module(module_path)
        return getattr(module, class_name)
    except (ImportError, AttributeError) as e:
        msg = "Could not import auth/permission class '{}'. {}: {}.".format(
            val,
            e.__class__.__name__,
            e)
        raise ImportError(msg)


def secure(view):
    """Set an auth decorator applied for views.

    If DEBUG is on, we serve the view without authenticating.

    Default is 'django.contrib.auth.decorators.login_required'.
    Can also be 'django.contrib.admin.views.decorators.staff_member_required'
    or a custom decorator.
    """
    auth_decorator = import_class(app_settings.AUTH_DECORATOR)
    return (
        view if settings.DEBUG
        else method_decorator(auth_decorator, name='dispatch')(view)
    )


def build_context(context, request, chart_id='', template_id=''):
    """Build context object to pass to the chartwerk editor."""
    def urlize(path):
        uri = os.path.join(reverse('chartwerk_home'), path)
        return '{}://{}{}'.format(request.scheme, request.get_host(), uri)
    context['user'] = request.user.username or 'DEBUGGER'
    context['chart_id'] = chart_id
    context['template_id'] = template_id
    context['chart_api'] = urlize('api/charts/')
    context['template_api'] = urlize('api/templates/')
    context['embed_api'] = urlize('api/embeds/')
    context['template_tags_api'] = urlize('api/template-property/')
    context['oembed'] = app_settings.OEMBED
    context['color_schemes'] = json.dumps(app_settings.COLOR_SCHEMES)
    return context


@secure
class Home(TemplateView):
    template_name = 'chartwerk/home.html'


@secure
class Browse(ListView):
    context_object_name = 'charts'
    template_name = 'chartwerk/browse.html'
    queryset = Chart.objects.all().order_by('-pk')


@secure
class Start(ListView):
    context_object_name = 'templates'
    template_name = 'chartwerk/start.html'
    queryset = Template.objects.all().order_by('-pk')


@secure
class MyWerk(ListView):
    context_object_name = 'charts'
    template_name = 'chartwerk/myWerk.html'
    queryset = Chart.objects.all().order_by('-pk')

    def get_queryset(self):
        user = self.request.user.username or 'DEBUGGER'
        return Chart.objects.filter(creator=user)

    def get_context_data(self, **kwargs):
        context = super(MyWerk, self).get_context_data(**kwargs)
        context['user'] = self.request.user or 'DEBUGGER'
        return context


@secure
class ChartDetail(DetailView):
    model = Chart
    template_name = 'chartwerk/editor.html'

    def get_context_data(self, **kwargs):
        context = super(ChartDetail, self).get_context_data(**kwargs)

        return build_context(
            context,
            self.request,
            chart_id=self.object.slug
        )


@secure
class TemplateDetail(DetailView):
    model = Template
    template_name = 'chartwerk/editor.html'

    def get_context_data(self, **kwargs):
        context = super(TemplateDetail, self).get_context_data(**kwargs)

        return build_context(
            context,
            self.request,
            template_id=self.object.slug
        )


class JSONResponseMixin(object):
    def render_to_json_response(self, context, **response_kwargs):
        return JsonResponse(self.get_data(context), **response_kwargs)

    def get_data(self, context):
        return context


class CustomModelViewSet(viewsets.ModelViewSet):
    permission_classes = tuple(
        import_class(permission) for permission in
        app_settings.API_PERMISSION_CLASSES
    )
    authentication_classes = tuple(
        import_class(auth) for auth in
        app_settings.API_AUTHENTICATION_CLASSES
    )


class ChartViewSet(CustomModelViewSet):
    queryset = Chart.objects.all()
    serializer_class = ChartSerializer
    lookup_field = 'slug'


class TemplateViewSet(CustomModelViewSet):
    queryset = Template.objects.all()
    serializer_class = TemplateSerializer
    lookup_field = 'slug'


class TemplatePropertyViewSet(CustomModelViewSet):
    pagination_class = None
    queryset = TemplateProperty.objects.all()
    serializer_class = TemplatePropertySerializer


class FinderQuestionViewSet(CustomModelViewSet):
    queryset = FinderQuestion.objects.all()
    serializer_class = FinderQuestionSerializer


class ChartEmbedViewSet(CustomModelViewSet):
    queryset = Chart.objects.all()
    serializer_class = ChartEmbedSerializer
    lookup_field = 'slug'

    def get_serializer_context(self):
        return {
            'size': self.request.query_params.get('size', 'double')
        }


def oEmbed(request):
    """Return an oEmbed json response."""
    if 'url' not in request.GET or not request.GET.get('url'):
        return HttpResponseBadRequest('url parameter is required.')

    url = request.GET.get('url')
    size = request.GET.get('size', 'double')

    path = urlparse(url).path

    try:
        chart_kwargs = resolve(path).kwargs
    except NoReverseMatch:
        for pattern in app_settings.OEMBED_EXTRA_PATTERNS:
            chart_kwargs = re.search(pattern, path[1:])

            if chart_kwargs is not None:
                chart_kwargs = chart_kwargs.groupdict()
                break

    if chart_kwargs is None:
        return HttpResponseNotFound(
            '"%s" did not match any supported oEmbed URL patterns.' % url
        )

    try:
        chart = Chart.objects.get(**chart_kwargs)
        return JsonResponse(chart.oembed(size=size))
    except Chart.DoesNotExist:
        return HttpResponseNotFound(
            'Chart matching "%s" not found.' % chart_kwargs
        )



from chartwerk.models import Chart, FinderQuestion, Template, TemplateProperty
from django.contrib import admin


class ChartAdmin(admin.ModelAdmin):
    readonly_fields = (
        'author',
        'created',
        'updated',
    )
    list_display = (
        'slug',
        'title',
        'creator',
    )


class TemplateAdmin(admin.ModelAdmin):
    readonly_fields = (
        'author',
        'created',
        'updated',
        'description',
    )
    list_display = (
        'slug',
        'title',
        'creator',
    )


admin.site.register(Chart, ChartAdmin)
admin.site.register(Template, TemplateAdmin)
admin.site.register(TemplateProperty)
admin.site.register(FinderQuestion)



from chartwerk.celery import aws, github, slack
from chartwerk.models import Chart, Template
from django.db.models.signals import post_save
from django.dispatch import receiver


@receiver(post_save, sender=Chart)
def post_chart(sender, instance, **kwargs):
    if kwargs.get('created'):
        instance.creator = instance.author
        instance.save()
        slack.delay(instance.pk)
    aws.delay(instance.pk)


@receiver(post_save, sender=Template)
def post_template(sender, instance, **kwargs):
    if kwargs.get('created', False):
        instance.creator = instance.author
        instance.save()
    github.delay(instance.pk)



"""Celery task for sending Slack notifications."""
from __future__ import absolute_import

import logging
import os
import time

from slacker import Slacker

from celery import shared_task
from chartwerk.conf import settings as app_settings
from chartwerk.models import Chart, Template
from django.contrib.staticfiles.templatetags.staticfiles import static
from django.urls import reverse

logger = logging.getLogger(__name__)


def get_template_icon(template_title):
    """Get template's icon."""
    template = Template.objects.filter(title=template_title).first()
    if template is not None and template.icon:
        return template.icon.url
    return ''


def get_slack_user(slack, email):
    """Get a Slack user by email."""
    users = slack.users.list().body['members']
    for u in users:
        if u.get('profile', {}).get('email', '') == email:
            return '<@{}|{}>'.format(
                u.get('id'),
                u.get('name', 'stranger')
            )
    return 'a stranger'


@shared_task
def notify_slack(pk):
    """Send slack notification."""
    if not app_settings.SLACK_TOKEN:
        return

    werk = Chart.objects.get(pk=pk)

    slack = Slacker(app_settings.SLACK_TOKEN)

    try:
        chart_url = os.path.join(
            app_settings.DOMAIN,
            reverse('chartwerk_chart', kwargs={'slug': werk.slug})[1:]
        )
        attachmentData = [{
            'fallback': '{} created a new chart, "{}" at {}'.format(
                get_slack_user(slack, werk.author),
                werk.title,
                chart_url
            ),
            'color': '#C91507',
            'pretext': 'New chart by {}'.format(
                get_slack_user(slack, werk.author)
            ),
            'title': werk.title,
            'title_link': chart_url,
            'text': werk.data['template']['title'],
            'thumb_url': get_template_icon(
                werk.data['template']['title']
            ),
            'footer': 'chartwerk',
            'footer_icon': os.path.join(
                app_settings.DOMAIN,
                static('chartwerk/img/chartwerk_30.png')[1:]
            ),
            'ts': int(time.time())
        }]

        bot_icon = os.path.join(
            app_settings.DOMAIN,
            static('chartwerk/img/chartwerk_60.png')[1:]
        )

        slack.chat.post_message(
            app_settings.SLACK_CHANNEL,
            '',
            attachments=attachmentData,
            as_user=False,
            icon_url=bot_icon,
            username='Chartwerk'
        )
    except Exception:
        logging.exception("Slack notification error")



"""Celery task to commit templates to GitHub."""
from __future__ import absolute_import

import json
import logging

from celery import shared_task
from chartwerk.conf import settings as app_settings
from chartwerk.models import Template
from github import Github, GithubException, UnknownObjectException

logger = logging.getLogger(__name__)


def get_github_credentials():
    """Get the credentials kwargs we're going to pass to Github().

    Prefers a personal token over a username/password.
    """
    if app_settings.GITHUB_TOKEN:
        return dict(login_or_token=app_settings.GITHUB_TOKEN)
    else:
        return dict(
            login_or_token=app_settings.GITHUB_USER,
            password=app_settings.GITHUB_PASSWORD
        )


def get_repository():
    """Get the GitHub repo specified in settings or the default.

    If the repo doesn't exist, try to create it.
    """
    try:
        g = Github(**get_github_credentials())

        if app_settings.GITHUB_ORG:
            user = g.get_organization(app_settings.GITHUB_ORG)
        else:
            user = g.get_user()

        try:
            return user.get_repo(app_settings.GITHUB_REPO)
        except UnknownObjectException:
            logging.info("Creating repository {}".format(
                app_settings.GITHUB_REPO
            ))
            return user.create_repo(app_settings.GITHUB_REPO)
    except GithubException:
        logging.exception("Unable to configure Github connection.")


def commit_script(path, script):
    """Commit script string to GitHub repo."""
    repository = get_repository()

    try:
        sha = repository.get_contents(path).sha
        repository.update_file(
            path=path,
            message='Template update',
            content=script,
            sha=sha
        )
    except GithubException:
        repository.create_file(
            path=path,
            message='Template initial commit',
            content=script
        )
        logging.info("Initial commit of new file {}".format(path))


@shared_task
def commit_template(pk):
    """Commit template scripts to GitHub."""
    if not app_settings.GITHUB_REPO:
        return

    template = Template.objects.get(pk=pk)

    commit_script(
        '/{}/draw.js'.format(template.slug),
        template.data['scripts']['draw']
    )
    commit_script(
        '/{}/helper.js'.format(template.slug),
        template.data['scripts']['helper']
    )
    commit_script(
        '/{}/chart.html'.format(template.slug),
        template.data['scripts']['html']
    )
    commit_script(
        '/{}/styles.css'.format(template.slug),
        template.data['scripts']['styles']
    )
    commit_script(
        '/{}/template.json'.format(template.slug),
        json.dumps(template.data, sort_keys=True, indent=4)
    )



import json
import os
import random
import string
from datetime import datetime

from chartwerk.conf import settings as app_settings
from django import template
from django.contrib.postgres.fields import JSONField
from django.db import models
from django.urls import reverse
from django.utils.text import slugify
from uuslug import uuslug


def template_icon_upload_path(instance, filename):
    fileName, fileExtension = os.path.splitext(filename)
    return os.path.join(
        'templates/icons',
        '{}-{}{}'.format(
            slugify(instance.title),
            datetime.now().strftime('%Y%m%d'),
            fileExtension
        )
    )


class Chartwerk(models.Model):
    slug = models.SlugField(
        max_length=250,
        unique=True,
        blank=True,
        null=True
    )
    title = models.CharField(max_length=240)
    data = JSONField()
    created = models.DateTimeField(auto_now_add=True, editable=False)
    updated = models.DateTimeField(auto_now=True, editable=False)
    # User who last committed chart
    author = models.CharField(
        max_length=250,
        blank=True,
        null=True,
    )
    # User who first committed chart
    creator = models.CharField(
        max_length=250,
        blank=True,
        null=True,
    )

    class Meta:
        abstract = True


class Chart(Chartwerk):
    embed_data = JSONField(blank=True, null=True)

    def save(self, *args, **kwargs):
        if not self.slug:
            char_set = string.ascii_uppercase + \
                string.ascii_lowercase + \
                string.digits
            uuid = ''.join(random.choice(char_set) for _ in range(8))
            self.slug = uuslug(
                uuid,
                instance=self,
                max_length=8,
                separator=''
            )
        super(Chart, self).save(*args, **kwargs)

    def __str__(self):
        return "{} - {} by {}".format(self.slug, self.title, self.author)

    def get_absolute_url(self):
        return reverse('chartwerk_chart', kwargs=dict(slug=self.slug))

    def get_embed_code(self, size='double'):
        """Return the embed code for the chart."""
        def simple_string(split_string):
            """Return a string stripped of extra whitespace."""
            return ' '.join(split_string.split())

        template_object = template.Template(app_settings.EMBED_TEMPLATE)
        template_context_object = app_settings.EMBED_TEMPLATE_CONTEXT(self)
        template_context_object.update({
            'size': size,
            'id': self.slug,
            'dimensions': json.dumps(self.embed_data).replace('"', '&quot;'),
        })
        return simple_string(
            template_object.render(template.Context(
                template_context_object
            ))
        )

    def oembed(self, size='double'):
        return {
            "version": "1.0",
            "url": os.path.join(
                app_settings.DOMAIN,
                self.get_absolute_url()[1:],
            ),
            "title": self.title,
            "provider_url": os.path.join(
                app_settings.DOMAIN,
                reverse('chartwerk_home')[1:]
            ),
            "provider_name": "Chartwerk",
            "author_name": self.creator,
            "chart_id": self.slug,
            "type": "rich",
            "size": size,
            "width": self.embed_data['double']['width'] or "",
            "height": self.embed_data['double']['height'] or "",
            "single_width": self.embed_data['single']['width'] or "",
            "single_height": self.embed_data['single']['height'] or "",
            "html": self.get_embed_code(size),
        }


class Template(Chartwerk):
    def chart_count(self):
        return Chart.objects.filter(data__template__title=self.title).count()

    icon = models.ImageField(
        upload_to=template_icon_upload_path, blank=True, null=True)
    description = models.TextField(blank=True, null=True)

    def save(self, *args, **kwargs):
        self.slug = uuslug(self.title, instance=self)
        super(Template, self).save(*args, **kwargs)

    def __str__(self):
        return self.slug


class TemplateProperty(models.Model):
    slug = models.CharField(max_length=250, unique=True)
    property = models.CharField(max_length=250, unique=True)
    template = models.ManyToManyField(
        Template,
        related_name="template_properties"
    )
    finder = models.ManyToManyField(
        'FinderQuestion',
        blank=True,
        related_name="template_properties"
    )

    def save(self, *args, **kwargs):
        self.slug = uuslug(self.property, instance=self)
        super(TemplateProperty, self).save(*args, **kwargs)

    def __str__(self):
        return self.property

    class Meta:
        verbose_name_plural = "Template properties"


class FinderQuestion(models.Model):
    question = models.CharField(max_length=250)
    order = models.PositiveSmallIntegerField(unique=True)

    def __str__(self):
        return self.question



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-06-01 18:02
from __future__ import unicode_literals

import chartwerk.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('chartwerk', '0002_loadfreetemplates'),
    ]

    operations = [
        migrations.AlterField(
            model_name='template',
            name='icon',
            field=models.ImageField(blank=True, null=True, upload_to=chartwerk.models.template_icon_upload_path),
        ),
    ]



from chartwerk.models import Chart, FinderQuestion, Template, TemplateProperty
from rest_framework import serializers


class ChartSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='chart-detail',
        lookup_field='slug'
    )

    class Meta:
        model = Chart
        fields = '__all__'


class ReverseTemplatePropertySerializer(serializers.ModelSerializer):
    class Meta:
        model = TemplateProperty
        fields = ('slug',)
        extra_kwargs = {
            'id': {'read_only': False},
            'slug': {'validators': []},
        }


class TemplateSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='template-detail',
        lookup_field='slug'
    )
    template_properties = ReverseTemplatePropertySerializer(many=True)

    class Meta:
        model = Template
        fields = '__all__'

    def create(self, validated_data):
        properties_data = validated_data.pop('template_properties')
        template = Template.objects.create(**validated_data)
        for property_ in properties_data:
            try:
                obj = TemplateProperty.objects.get(slug=property_.get('slug'))
            except TemplateProperty.DoesNotExist:
                continue

            obj.template.add(template)
        return template

    def update(self, instance, validated_data):
        properties_data = validated_data.pop('template_properties')
        # UPDATE instance
        for k, v in validated_data.items():
            setattr(instance, k, v)
        instance.save()
        # SYNC template tags
        tags_list = []
        # ADD new template tags
        for property_ in properties_data:
            slug = property_.get('slug')

            try:
                obj = TemplateProperty.objects.get(slug=slug)
            except TemplateProperty.DoesNotExist:
                continue

            obj.template.add(instance)
            tags_list.append(slug)
        # REMOVE any template tags not in the new set
        drop_props = instance.template_properties.exclude(slug__in=tags_list)
        for prop in drop_props:
            prop.template.remove(instance)
        return instance


class FinderQuestionSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = FinderQuestion
        fields = '__all__'


class TemplatePropertySerializer(serializers.HyperlinkedModelSerializer):
    finder = serializers.HyperlinkedRelatedField(
        many=True,
        read_only=True,
        view_name='finderquestion-detail'
    )
    template = serializers.SlugRelatedField(
        many=True,
        read_only=True,
        slug_field='slug'
    )

    class Meta:
        model = TemplateProperty
        fields = '__all__'


class ReverseTemplatePropertySerializer(serializers.ModelSerializer):
    class Meta:
        model = Chart
        fields = ('slug',)
        extra_kwargs = {
            'id': {'read_only': False},
            'slug': {'validators': []},
        }


class ChartEmbedSerializer(serializers.ModelSerializer):
    embed_code = serializers.SerializerMethodField()

    class Meta:
        model = Chart
        fields = ('slug', 'embed_code')

    def get_embed_code(self, obj):
        return obj.get_embed_code(self.context.get('size'))



import json
import os

from chartwerk.models import Template
from django import template
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.staticfiles.templatetags.staticfiles import static

DOMAIN = settings.CHARTWERK_DOMAIN

register = template.Library()


@register.filter
def jsonify(value):
    return json.dumps(value)


@register.filter
def user_by_email(email):
    if email is None:
        return 'DEBUGGER'
    user = User.objects.filter(email=email).first()
    if user:
        if user.first_name != '' and user.last_name != '':
            return '{} {}'.format(
                user.first_name,
                user.last_name
            )
    else:
        return email


@register.filter
def get_icon(title):
    default = os.path.join(
        DOMAIN,
        static('chartwerk/img/chartwerk_100.png')[1:]
    )
    if title is None:
        return default
    template = Template.objects.filter(title=title).first()
    if template and template.icon:
        return Template.objects.filter(title=title).first().icon.url
    else:
        return default



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-05-24 02:30
from __future__ import unicode_literals

import chartwerk.models
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Chart',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(blank=True, max_length=250, null=True, unique=True)),
                ('title', models.CharField(max_length=240)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField()),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('author', models.CharField(blank=True, max_length=250, null=True)),
                ('creator', models.CharField(blank=True, max_length=250, null=True)),
                ('embed_data', django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='FinderQuestion',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('question', models.CharField(max_length=250)),
                ('order', models.PositiveSmallIntegerField(unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='Template',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(blank=True, max_length=250, null=True, unique=True)),
                ('title', models.CharField(max_length=240)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField()),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('author', models.CharField(blank=True, max_length=250, null=True)),
                ('creator', models.CharField(blank=True, max_length=250, null=True)),
                ('icon', models.ImageField(blank=True, null=True)),
                ('description', models.TextField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TemplateProperty',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.CharField(max_length=250, unique=True)),
                ('property', models.CharField(max_length=250, unique=True)),
                ('finder', models.ManyToManyField(blank=True, related_name='template_properties', to='chartwerk.FinderQuestion')),
                ('template', models.ManyToManyField(related_name='template_properties', to='chartwerk.Template')),
            ],
            options={
                'verbose_name_plural': 'Template properties',
            },
        ),
    ]



# Imports from standard libraries.  # NOQA
import os

from setuptools import find_packages, setup

REPO_URL = 'https://github.com/DallasMorningNews/django-chartwerk/'

PYPI_VERSION = '0.5.6'

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


setup(
    name='django-chartwerk',
    version=PYPI_VERSION,
    packages=find_packages(),
    include_package_data=True,
    license='AGPLv3',
    description=(
        'A Django application to manage, create and share Chartwerk charts.'
    ),
    long_description=README,
    url=REPO_URL,
    download_url='{repo_url}archive/{version}.tar.gz'.format(**{
        'repo_url': REPO_URL,
        'version': PYPI_VERSION,
    }),
    author='Jon McClure / The Dallas Morning News',
    author_email='newsapps@dallasnews.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'boto3>=1.4.0',
        'celery>=4.0.0',
        'Django>=1.9',
        'django-uuslug>=1.1.8',
        'djangorestframework>=2.4',
        'Pillow',
        'psycopg2>=2.5.4',
        'PyGithub>=1.29',
        'slacker>=0.9',
    ],
)



"""
This module manages app configuration.

Some settings are required and raise errors if not set.
Other settings have defaults. Optional settigns default to None.
"""

import os

from django.conf import settings as project_settings
from django.templatetags.static import static


class ChartwerkConfigError(Exception):
    """Raised when required config is not present."""

    pass


class Settings:
    pass


#####################
# REQUIRED SETTINGS #
#####################


Settings.DOMAIN = getattr(project_settings, 'CHARTWERK_DOMAIN', None)

if not Settings.DOMAIN:
    raise ChartwerkConfigError('You haven\'t set the CHARTWERK_DOMAIN \
variable. Set it in your project settings.')

Settings.AWS_BUCKET = getattr(project_settings, 'CHARTWERK_AWS_BUCKET', None)

if not Settings.AWS_BUCKET:
    raise ChartwerkConfigError('You haven\'t set the CHARTWERK_AWS_BUCKET \
variable. Set it either in your project settings.')

Settings.AWS_ACCESS_KEY_ID = getattr(
    project_settings, 'CHARTWERK_AWS_ACCESS_KEY_ID', None)

if not Settings.AWS_ACCESS_KEY_ID:
    raise ChartwerkConfigError('You haven\'t set the CHARTWERK_AWS_ACCESS_KEY_ID \
variable. Set it in your project settings.')

Settings.AWS_SECRET_ACCESS_KEY = getattr(
    project_settings, 'CHARTWERK_AWS_SECRET_ACCESS_KEY', None)

if not Settings.AWS_SECRET_ACCESS_KEY:
    raise ChartwerkConfigError('You haven\'t set the CHARTWERK_AWS_SECRET_ACCESS_KEY \
variable. Set it in your project settings.')


########################
# SETTINGS W/ DEFAULTS #
########################

DEFAULT_TEMPLATE_PATH = os.path.join(
    os.path.dirname(__file__),
    'templates/chartwerk/embed.html'
)
with open(DEFAULT_TEMPLATE_PATH, 'r') as template_file:
    DEFAULT_TEMPLATE = template_file.read()

Settings.EMBED_TEMPLATE = getattr(
    project_settings,
    'CHARTWERK_EMBED_TEMPLATE',
    DEFAULT_TEMPLATE
)

EMBED_TEMPLATE_CONTEXT = getattr(
    project_settings, 'CHARTWERK_EMBED_TEMPLATE_CONTEXT',
    lambda chart: {
        'chart_path': 'http://www.somesite.com/path/to/charts/',
        'embed_script': os.path.join(
            Settings.DOMAIN,
            static('chartwerk/js/main-embed.bundle.js'),
        ),
    }
)
if not callable(EMBED_TEMPLATE_CONTEXT):
    raise ChartwerkConfigError('CHARTWERK_EMBED_TEMPLATE_CONTEXT must be a \
function that accepts one parameter, the chart object.')

Settings.EMBED_TEMPLATE_CONTEXT = staticmethod(EMBED_TEMPLATE_CONTEXT)

Settings.OEMBED = getattr(project_settings, 'CHARTWERK_OEMBED', False)

Settings.OEMBED_EXTRA_PATTERNS = getattr(
    project_settings, 'CHARTWERK_OEMBED_EXTRA_PATTERNS', [])

Settings.AWS_PATH = getattr(project_settings, 'CHARTWERK_AWS_PATH', 'charts')

Settings.CACHE_HEADER = getattr(
    project_settings, 'CHARTWERK_CACHE_HEADER', 'max-age=300')

Settings.AUTH_DECORATOR = getattr(
        project_settings,
        'CHARTWERK_AUTH_DECORATOR',
        'django.contrib.auth.decorators.login_required'
    )

Settings.API_AUTHENTICATION_CLASSES = getattr(
        project_settings,
        'CHARTWERK_API_AUTHENTICATION_CLASSES',
        ('rest_framework.authentication.SessionAuthentication',)
    )

Settings.API_PERMISSION_CLASSES = getattr(
        project_settings,
        'CHARTWERK_API_PERMISSION_CLASSES',
        ('rest_framework.permissions.IsAuthenticatedOrReadOnly',)
    )

Settings.COLOR_SCHEMES = getattr(
    project_settings,
    'CHARTWERK_COLOR_SCHEMES',
    {}
)

#####################
# OPTIONAL SETTINGS #
#####################

Settings.GITHUB_REPO = None

Settings.GITHUB_TOKEN = getattr(
    project_settings, 'CHARTWERK_GITHUB_TOKEN', None)

Settings.GITHUB_USER = getattr(
    project_settings, 'CHARTWERK_GITHUB_USER', None)

Settings.GITHUB_ORG = getattr(
    project_settings, 'CHARTWERK_GITHUB_ORG', None)

Settings.GITHUB_PASSWORD = getattr(
    project_settings, 'CHARTWERK_GITHUB_PASSWORD', None)

# If setting USER or PASSWORD, should set USER and PASSWORD
if not Settings.GITHUB_TOKEN:
    if Settings.GITHUB_PASSWORD and not Settings.GITHUB_USER:
        raise ChartwerkConfigError('You set the CHARTWERK_GITHUB_PASSWORD \
variable, but you haven\'t set the CHARTWERK_GITHUB_USER variable. \
Set it in your project settings or set CHARTWERK_GITHUB_TOKEN.')

    if Settings.GITHUB_USER and not Settings.GITHUB_PASSWORD:
        raise ChartwerkConfigError('You set the CHARTWERK_GITHUB_USER \
variable, but you haven\'t set the CHARTWERK_GITHUB_PASSWORD variable. \
Set it in your project settings or set CHARTWERK_GITHUB_TOKEN.')

if Settings.GITHUB_TOKEN or Settings.GITHUB_USER:

    Settings.GITHUB_REPO = getattr(
            project_settings,
            'CHARTWERK_GITHUB_REPO',
            'chartwerk_chart-templates'
        )


Settings.SLACK_CHANNEL = getattr(
    project_settings, 'CHARTWERK_SLACK_CHANNEL', '#chartwerk')

Settings.SLACK_TOKEN = getattr(
    project_settings, 'CHARTWERK_SLACK_TOKEN', None)

settings = Settings



from chartwerk.views import (Browse, ChartDetail, ChartEmbedViewSet,
                             ChartViewSet, FinderQuestionViewSet, Home, MyWerk,
                             Start, TemplateDetail, TemplatePropertyViewSet,
                             TemplateViewSet, oEmbed)
from django.conf.urls import include, url
from rest_framework import routers

router = routers.DefaultRouter()
router.register(r'charts', ChartViewSet)
router.register(r'templates', TemplateViewSet)
router.register(r'embeds', ChartEmbedViewSet)
router.register(r'finder-question', FinderQuestionViewSet)
router.register(r'template-property', TemplatePropertyViewSet)


urlpatterns = [
    url(r'^$', Home.as_view(), name='chartwerk_home'),
    url(r'^start/$', Start.as_view(), name='chartwerk_start'),
    url(r'^browse/$', Browse.as_view(), name='chartwerk_browse'),
    url(r'^myWerk/$', MyWerk.as_view(), name='chartwerk_mywerk'),
    url(r'^chart/(?P<slug>[-\w]+)/$', ChartDetail.as_view(),
        name='chartwerk_chart'),
    url(r'^template/(?P<slug>[-\w]+)/$', TemplateDetail.as_view(),
        name='chartwerk_template'),
    url(r'^api/', include(router.urls)),
    url(r'^api/oembed/$', oEmbed),
]
